/*
 * SSV HardWareAccess driver module and userspace functions for DNP/5370
 * Author  : ENE@SSV <info@dilnetpc.com> <eddso@gmx.net>
 * Date    : 04.04.2007
 * Version : 1.00
 * License : GPL
 */

#ifndef _SSVHWA_C
#define _SSVHWA_C

#ifdef __KERNEL__

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <asm/blackfin.h>
#include <asm/uaccess.h>
#include "ssvhwa.h"

MODULE_AUTHOR		("ENE@SSV <info@dilnetpc.com> <eddso@gmx.net>");
MODULE_DESCRIPTION	("SSV HardWareAccess driver for DNP/5370");
MODULE_LICENSE		("GPL");

/* ioctl method - ioctl(...) */
static int ssvhwa_ioctl(struct inode *inode, struct file *file,unsigned int cmd, unsigned long arg){
    struct ssvhwa_struct *ssvhwa = (struct ssvhwa_struct *)arg;
    int	returncode = 0;
    u32 addr, data = 0;
    u8  size;

	switch (cmd) {
	    case SSVHWA_READ:
		get_user(addr, &ssvhwa->addr);
		get_user(size, &ssvhwa->size);
		switch (size) {
		    case 16:
			data = bfin_read16(addr);
			break;
		    case 32:
			data = bfin_read32(addr);
			break;
		    default:
			returncode = -EINVAL;
			break;
		}
		put_user(data, &ssvhwa->data);
		break;

	    case SSVHWA_WRITE:
		get_user(addr, &ssvhwa->addr);
		get_user(data, &ssvhwa->data);
		get_user(size, &ssvhwa->size);
		switch (size) {
		    case 16:
			bfin_write16(addr,data);
			break;
		    case 32:
			bfin_write32(addr,data);
			break;
		    default:
			returncode = -EINVAL;
			break;
		}
		break;

	    case SSVHWA_INFO:
		addr = SSVHWA_MAGIC;
		data = SSVHWA_VERSION;
		put_user(addr, &ssvhwa->addr);
		put_user(data, &ssvhwa->data);
		break;

	    default:
		/* unknown IOCTL */
		returncode = -EINVAL;
		break;
	}
    return returncode;
}

/* open method - device = open("/dev/ssvhwa", O_RDWR) */
static int ssvhwa_open(struct inode *inode, struct file *file){
    try_module_get(THIS_MODULE);
	return 0;
}

/* close method - close(device)... */
static int ssvhwa_release(struct inode *inode, struct file *file){
    module_put(THIS_MODULE);
	return 0;
}

static struct file_operations ssvhwa_fops = {
    .open     = ssvhwa_open,
    .release  = ssvhwa_release,
    .ioctl    = ssvhwa_ioctl,
};

/* load kernel module - insmod ssvhwa.ko */
static int __init ssvhwa_init(void){
    /* register character device handler */
	if (register_chrdev(SSVHWA_MAJOR,"ssvhwa",&ssvhwa_fops)) {
		printk(KERN_ERR "ssvhwa: unable to get major %d\n", SSVHWA_MAJOR);
		return -EIO;
    }
    printk(KERN_INFO "ssvhwa module installed.\n");
    return 0;
}

/* unload kernel module - rmmod ssvhwa */
static void __exit ssvhwa_cleanup(void){
    /* unregister character device handler */
    unregister_chrdev(SSVHWA_MAJOR, "ssvhwa");
    printk(KERN_INFO "ssvhwa module removed.\n");
}

module_init(ssvhwa_init);
module_exit(ssvhwa_cleanup);


#else


#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/ioctl.h>

#include "ssvhwa.h"

int ssvhwa_device = 0;

int ssvhwa_open(void){
struct ssvhwa_struct info;

    /* check for reopen */
    if (ssvhwa_device > 0)
	return(0);

    /* open ssvhwa device driver */
    do {
	ssvhwa_device = open("/dev/ssvhwa", O_RDWR);
    } while ((ssvhwa_device == 0) ||
	     (ssvhwa_device == 1) ||
	     (ssvhwa_device == 2));

    if (ssvhwa_device < 0) {
		/* one more try - create device file and open */

		/* WARNING: create without check is a security risk - for uClinux ok */
		mkdir("/tmp/dev", 0700);
		mknod("/tmp/dev/ssvhwa", (0600 | S_IFCHR), (SSVHWA_MAJOR << 8));

		do {
		    ssvhwa_device = open("/tmp/dev/ssvhwa", O_RDWR);
		} while ((ssvhwa_device == 0) ||
			 (ssvhwa_device == 1) ||
			 (ssvhwa_device == 2));

		if (ssvhwa_device < 0) {
		    ssvhwa_device = 0;
		    return(-1);
		}
    }

    if (ioctl(ssvhwa_device, SSVHWA_INFO, &info) < 0) {
		close(ssvhwa_device);
		ssvhwa_device = 0;
		return(-2);
    }

    if ((info.addr != SSVHWA_MAGIC) || (info.data != SSVHWA_VERSION)) {
		close(ssvhwa_device);
		ssvhwa_device = 0;
		return(-3);
    }
    return(0);
}

void ssvhwa_close(void){
    /* close ssvhwa device driver */
    if (ssvhwa_device > 0)
	close(ssvhwa_device);

    ssvhwa_device = 0;
}

unsigned short ssvhwa_read16(unsigned long addr){
struct ssvhwa_struct hw;

    if (!ssvhwa_device)
	return(0);

    /* read hardware address - 16 bit */
    hw.addr = addr;
    hw.size = 16;
    ioctl(ssvhwa_device, SSVHWA_READ, &hw);
    return(hw.data);
}

unsigned long ssvhwa_read32(unsigned long addr){
struct ssvhwa_struct hw;

    if (!ssvhwa_device)
	return(0);

    /* read hardware address - 32 bit */
    hw.addr = addr;
    hw.size = 32;
    ioctl(ssvhwa_device, SSVHWA_READ, &hw);
    return(hw.data);
}

void ssvhwa_write16(unsigned long addr, unsigned short data){
struct ssvhwa_struct hw;

    if (!ssvhwa_device)
	return;

    /* write hardware address - 16 bit */
    hw.addr = addr;
    hw.data = data;
    hw.size = 16;
    ioctl(ssvhwa_device, SSVHWA_WRITE, &hw);
}

void ssvhwa_write32(unsigned long addr, unsigned long data){
struct ssvhwa_struct hw;

    if (!ssvhwa_device)
	return;

    /* write hardware address - 32 bit */
    hw.addr = addr;
    hw.data = data;
    hw.size = 32;
    ioctl(ssvhwa_device, SSVHWA_WRITE, &hw);
}

#endif
#endif
