/*
 * SSV HardWareAccess driver module and userspace functions for DNP/5370
 * Author  : ENE@SSV <info@dilnetpc.com> <eddso@gmx.net>
 * Date    : 04.04.2007
 * Version : 1.00
 * License : GPL
 */

#ifndef _SSVHWA_H
#define _SSVHWA_H

struct ssvhwa_struct
{
    unsigned long addr;
    unsigned long data;
    unsigned char size;
};

/* default for DNP/5370*/
#define SSVHWA_MAJOR 127

#define SSVHWA_IOCTL_BASE 0x7F
#define SSVHWA_INFO     _IOR(SSVHWA_IOCTL_BASE, 0, struct ssvhwa_struct)
#define SSVHWA_READ     _IOR(SSVHWA_IOCTL_BASE, 1, struct ssvhwa_struct)
#define SSVHWA_WRITE    _IOW(SSVHWA_IOCTL_BASE, 2, struct ssvhwa_struct)

#define SSVHWA_VERSION 0x20070404
#define SSVHWA_MAGIC   0x4d484131

#ifndef __KERNEL__
/* General Purpose I/O Port F (0xFFC00700 - 0xFFC007FF) for configure Port C								*/
#define PORTF_FER				0xFFC03200	/* Port F Function Enable Register (Alternate/Flag*)    */
#define PORTFIO					0xFFC00700	/* Port F I/O Pin State Specify Register                                */
#define PORTFIO_CLEAR			0xFFC00704	/* Port F I/O Peripheral Interrupt Clear Register               */
#define PORTFIO_SET				0xFFC00708	/* Port F I/O Peripheral Interrupt Set Register                 */
#define PORTFIO_TOGGLE			0xFFC0070C	/* Port F I/O Pin State Toggle Register                                 */
#define PORTFIO_MASKA			0xFFC00710	/* Port F I/O Mask State Specify Interrupt A Register   */
#define PORTFIO_MASKA_CLEAR		0xFFC00714	/* Port F I/O Mask Disable Interrupt A Register                 */
#define PORTFIO_MASKA_SET		0xFFC00718	/* Port F I/O Mask Enable Interrupt A Register                  */
#define PORTFIO_MASKA_TOGGLE	0xFFC0071C	/* Port F I/O Mask Toggle Enable Interrupt A Register   */
#define PORTFIO_MASKB			0xFFC00720	/* Port F I/O Mask State Specify Interrupt B Register   */
#define PORTFIO_MASKB_CLEAR		0xFFC00724	/* Port F I/O Mask Disable Interrupt B Register                 */
#define PORTFIO_MASKB_SET		0xFFC00728	/* Port F I/O Mask Enable Interrupt B Register                  */
#define PORTFIO_MASKB_TOGGLE	0xFFC0072C	/* Port F I/O Mask Toggle Enable Interrupt B Register   */
#define PORTFIO_DIR				0xFFC00730	/* Port F I/O Direction Register                                                */
#define PORTFIO_POLAR			0xFFC00734	/* Port F I/O Source Polarity Register                                  */
#define PORTFIO_EDGE			0xFFC00738	/* Port F I/O Source Sensitivity Register                               */
#define PORTFIO_BOTH			0xFFC0073C	/* Port F I/O Set on BOTH Edges Register                                */
#define PORTFIO_INEN			0xFFC00740	/* Port F I/O Input Enable Register                                     */
/* General Purpose I/O Port G (0xFFC01500 - 0xFFC015FF) for configure Port A and B							*/
#define PORTG_FER				0xFFC03204	/* Port G Function Enable Register (Alternate/Flag*)    */
#define PORTGIO					0xFFC01500	/* Port G I/O Pin State Specify Register                                */
#define PORTGIO_CLEAR			0xFFC01504	/* Port G I/O Peripheral Interrupt Clear Register               */
#define PORTGIO_SET				0xFFC01508	/* Port G I/O Peripheral Interrupt Set Register                 */
#define PORTGIO_TOGGLE			0xFFC0150C	/* Port G I/O Pin State Toggle Register                                 */
#define PORTGIO_MASKA			0xFFC01510	/* Port G I/O Mask State Specify Interrupt A Register   */
#define PORTGIO_MASKA_CLEAR		0xFFC01514	/* Port G I/O Mask Disable Interrupt A Register                 */
#define PORTGIO_MASKA_SET		0xFFC01518	/* Port G I/O Mask Enable Interrupt A Register                  */
#define PORTGIO_MASKA_TOGGLE	0xFFC0151C	/* Port G I/O Mask Toggle Enable Interrupt A Register   */
#define PORTGIO_MASKB			0xFFC01520	/* Port G I/O Mask State Specify Interrupt B Register   */
#define PORTGIO_MASKB_CLEAR		0xFFC01524	/* Port G I/O Mask Disable Interrupt B Register                 */
#define PORTGIO_MASKB_SET		0xFFC01528	/* Port G I/O Mask Enable Interrupt B Register                  */
#define PORTGIO_MASKB_TOGGLE	0xFFC0152C	/* Port G I/O Mask Toggle Enable Interrupt B Register   */
#define PORTGIO_DIR				0xFFC01530	/* Port G I/O Direction Register                                                */
#define PORTGIO_POLAR			0xFFC01534	/* Port G I/O Source Polarity Register                                  */
#define PORTGIO_EDGE			0xFFC01538	/* Port G I/O Source Sensitivity Register                               */
#define PORTGIO_BOTH			0xFFC0153C	/* Port G I/O Set on BOTH Edges Register                                */
#define PORTGIO_INEN			0xFFC01540	/* Port G I/O Input Enable Register   */
/* General Purpose I/O Port H (0xFFC01700 - 0xFFC017FF)*/
#define PORTH_FER				0xFFC03208	/* Port H Function Enable Register (Alternate/Flag*)    */
#define PORTHIO					0xFFC01700	/* Port H I/O Pin State Specify Register                                */
#define PORTHIO_CLEAR			0xFFC01704	/* Port H I/O Peripheral Interrupt Clear Register               */
#define PORTHIO_SET				0xFFC01708	/* Port H I/O Peripheral Interrupt Set Register                 */
#define PORTHIO_TOGGLE			0xFFC0170C	/* Port H I/O Pin State Toggle Register                                 */
#define PORTHIO_MASKA			0xFFC01710	/* Port H I/O Mask State Specify Interrupt A Register   */
#define PORTHIO_MASKA_CLEAR		0xFFC01714	/* Port H I/O Mask Disable Interrupt A Register                 */
#define PORTHIO_MASKA_SET		0xFFC01718	/* Port H I/O Mask Enable Interrupt A Register                  */
#define PORTHIO_MASKA_TOGGLE	0xFFC0171C	/* Port H I/O Mask Toggle Enable Interrupt A Register   */
#define PORTHIO_MASKB			0xFFC01720	/* Port H I/O Mask State Specify Interrupt B Register   */
#define PORTHIO_MASKB_CLEAR		0xFFC01724	/* Port H I/O Mask Disable Interrupt B Register                 */
#define PORTHIO_MASKB_SET		0xFFC01728	/* Port H I/O Mask Enable Interrupt B Register                  */
#define PORTHIO_MASKB_TOGGLE	0xFFC0172C	/* Port H I/O Mask Toggle Enable Interrupt B Register   */
#define PORTHIO_DIR				0xFFC01730	/* Port H I/O Direction Register                                                */
#define PORTHIO_POLAR			0xFFC01734	/* Port H I/O Source Polarity Register                                  */
#define PORTHIO_EDGE			0xFFC01738	/* Port H I/O Source Sensitivity Register                               */
#define PORTHIO_BOTH			0xFFC0173C	/* Port H I/O Set on BOTH Edges Register                                */
#define PORTHIO_INEN			0xFFC01740	/* Port H I/O Input Enable Register     */
/* CS */
#define EBIU_AMGCTL 	0xFFC00A00
#define EBIU_AMBCTL1 	0xFFC00A08
#define B3RDYEN			0x00010000	/* Bank 3 (B3) RDY Enable                           */
#define B3RDYPOL		0x00020000	/* B3 RDY Active High                               */
#define B3TT_1			0x00040000	/* B3 Transition Time (Read to Write) = 1 cycle     */
#define B3TT_2			0x00080000	/* B3 Transition Time (Read to Write) = 2 cycles    */
#define B3TT_3			0x000C0000	/* B3 Transition Time (Read to Write) = 3 cycles    */
#define B3TT_4			0x00000000	/* B3 Transition Time (Read to Write) = 4 cycles    */
#define B3ST_1			0x00100000	/* B3 Setup Time (AOE to Read/Write) = 1 cycle      */
#define B3ST_2			0x00200000	/* B3 Setup Time (AOE to Read/Write) = 2 cycles     */
#define B3ST_3			0x00300000	/* B3 Setup Time (AOE to Read/Write) = 3 cycles     */
#define B3ST_4			0x00000000	/* B3 Setup Time (AOE to Read/Write) = 4 cycles     */
#define B3HT_1			0x00400000	/* B3 Hold Time (~Read/Write to ~AOE) = 1 cycle     */
#define B3HT_2			0x00800000	/* B3 Hold Time (~Read/Write to ~AOE) = 2 cycles    */
#define B3HT_3			0x00C00000	/* B3 Hold Time (~Read/Write to ~AOE) = 3 cycles    */
#define B3HT_0			0x00000000	/* B3 Hold Time (~Read/Write to ~AOE) = 0 cycles    */
#define B3RAT_1			0x01000000	/* B3 Read Access Time = 1 cycle                    */
#define B3RAT_2			0x02000000	/* B3 Read Access Time = 2 cycles                   */
#define B3RAT_3			0x03000000	/* B3 Read Access Time = 3 cycles                   */
#define B3RAT_4			0x04000000	/* B3 Read Access Time = 4 cycles                   */
#define B3RAT_5			0x05000000	/* B3 Read Access Time = 5 cycles                   */
#define B3RAT_6			0x06000000	/* B3 Read Access Time = 6 cycles                                   */
#define B3RAT_7			0x07000000	/* B3 Read Access Time = 7 cycles                                   */
#define B3RAT_8			0x08000000	/* B3 Read Access Time = 8 cycles                                   */
#define B3RAT_9			0x09000000	/* B3 Read Access Time = 9 cycles                                   */
#define B3RAT_10		0x0A000000	/* B3 Read Access Time = 10 cycles                                  */
#define B3RAT_11		0x0B000000	/* B3 Read Access Time = 11 cycles                                  */
#define B3RAT_12		0x0C000000	/* B3 Read Access Time = 12 cycles                                  */
#define B3RAT_13		0x0D000000	/* B3 Read Access Time = 13 cycles                                  */
#define B3RAT_14		0x0E000000	/* B3 Read Access Time = 14 cycles                                  */
#define B3RAT_15		0x0F000000	/* B3 Read Access Time = 15 cycles                                  */
#define B3WAT_1			0x10000000	/* B3 Write Access Time = 1 cycle                                   */
#define B3WAT_2			0x20000000	/* B3 Write Access Time = 2 cycles                                  */
#define B3WAT_3			0x30000000	/* B3 Write Access Time = 3 cycles                                  */
#define B3WAT_4			0x40000000	/* B3 Write Access Time = 4 cycles                                  */
#define B3WAT_5			0x50000000	/* B3 Write Access Time = 5 cycles                                  */
#define B3WAT_6			0x60000000	/* B3 Write Access Time = 6 cycles                                  */
#define B3WAT_7			0x70000000	/* B3 Write Access Time = 7 cycles                                  */
#define B3WAT_8			0x80000000	/* B3 Write Access Time = 8 cycles                                  */
#define B3WAT_9			0x90000000	/* B3 Write Access Time = 9 cycles                                  */
#define B3WAT_10		0xA0000000	/* B3 Write Access Time = 10 cycles                                 */
#define B3WAT_11		0xB0000000	/* B3 Write Access Time = 11 cycles                                 */
#define B3WAT_12		0xC0000000	/* B3 Write Access Time = 12 cycles                                 */
#define B3WAT_13		0xD0000000	/* B3 Write Access Time = 13 cycles                                 */
#define B3WAT_14		0xE0000000	/* B3 Write Access Time = 14 cycles                                 */
#define B3WAT_15		0xF0000000	/* B3 Write Access Time = 15 cycles                                 */

#define PORT_A 		0x00FF 		/*Port_A*/
#define PORT_A0 	0x0001 		/*Port_A0 Socket pin 1 */
#define PORT_A1 	0x0002 		/*Port_A1 Socket pin 2 */
#define PORT_A2 	0x0004 		/*Port_A2 Socket pin 3 */
#define PORT_A3 	0x0008 		/*Port_A3 Socket pin 4*/
#define PORT_A4 	0x0010 		/*Port_A4 Socket pin 5 */
#define PORT_A5 	0x0020 		/*Port_A5 Socket pin 6 */
#define PORT_A6 	0x0040 		/*Port_A6 Socket pin 7*/
#define PORT_A7 	0x0080 		/*Port_A7 Socket pin 8 */
#define PORT_B 		0xFF00 		/*Port_B*/
#define PORT_B0 	0x0100 		/*Port_B0 Socket pin 9 */
#define PORT_B1 	0x0200 		/*Port_B0 Socket pin 10 */
#define PORT_B2 	0x0400 		/*Port_B0 Socket pin 11 */
#define PORT_B3 	0x0800 		/*Port_B0 Socket pin 12 */
#define PORT_B4 	0x1000 		/*Port_B0 Socket pin 13 */
#define PORT_B5 	0x2000 		/*Port_B0 Socket pin 14 */
#define PORT_B6 	0x4000 		/*Port_B0 Socket pin 15 */
#define PORT_B7 	0x8000 		/*Port_B0 Socket pin 16 */
#define PORT_C 		0x8380 		/*Port_C*/
#define PORT_C0 	0x0080 		/*Port_C0 Socket pin 17 */
#define PORT_C1 	0x0100 		/*Port_C0 Socket pin 18 */
#define PORT_C2 	0x0200 		/*Port_C0 Socket pin 19 */
#define PORT_C3 	0x8000 		/*Port_C0 Socket pin 20 */

extern int ssvhwa_device;
int ssvhwa_open(void);
void ssvhwa_close(void);
unsigned short ssvhwa_read16(unsigned long);
unsigned long ssvhwa_read32(unsigned long);
void ssvhwa_write16(unsigned long, unsigned short);
void ssvhwa_write32(unsigned long, unsigned long);

#endif
#endif
